@@ -155,7 +155,8 @@ typedef struct {
 	char 		*suffix;
 } cpu_freq_scale_t;
 
-static cpu_freq_scale_t cpu_freq_scale[] = {
+/* Different freq scaling rates */
+static const cpu_freq_scale_t cpu_freq_scale[] = {
 	{ 1e1,  1e0,  "Hz" },
 	{ 1e4,  1e3,  "KHz" },
 	{ 1e7,  1e6,  "MHz" },
@@ -796,7 +797,7 @@ static int stats_read(stats_t *const stats)
 	char buf[4096];
 	int i, j;
 
-	static stat_type indices[] = {
+	static const stat_type indices[] = {
 		CPU_USER, CPU_NICE, CPU_SYS, CPU_IDLE,
 		CPU_IOWAIT, CPU_IRQ, CPU_SOFTIRQ, CPU_CTXT,
 		CPU_INTR, CPU_PROCS_RUN, CPU_PROCS_BLK, -1
@@ -894,7 +895,7 @@ static bool stats_gather(
 	int i, j;
 	bool inaccurate = false;
 
-	static int indices[] = {
+	static const int indices[] = {
 		CPU_USER, CPU_NICE, CPU_SYS, CPU_IDLE,
 		CPU_IOWAIT, -1
 	};
@@ -1107,23 +1108,34 @@ static void stats_average_stddev_min_max(
 
 	for (j = 0; j < MAX_VALUES; j++) {
 		double total = 0.0;
+		double mant = 1.0;
+		int64_t expon = 0;
 
 		max->value[j] = -DBL_MAX;
 		min->value[j] = DBL_MAX;
-		geometric_mean->value[j] = 1.0;
 
 		for (valid = 0, i = 0; i < num; i++) {
 			if (!stats[i].inaccurate[j]) {
+				int e;
+				double f;
+
 				if (stats[i].value[j] > max->value[j])
 					max->value[j] = stats[i].value[j];
 				if (stats[i].value[j] < min->value[j])
 					min->value[j] = stats[i].value[j];
 				total += stats[i].value[j];
-				geometric_mean->value[j] *= stats[i].value[j];
+				
+				f = frexp(stats[i].value[j], &e);
+				mant *= f;
+				expon += e;
 				valid++;
 			}
 		}
 		if (valid) {
+			double inverse_n = 1.0 / (double)valid;
+
+			geometric_mean->value[j] = pow(mant, inverse_n) * 
+				pow(2.0, (double)expon / (double)valid);
 			average->value[j] = total / (double)valid;
 			total = 0.0;
 			for (i = 0; i < num; i++) {
@@ -1136,9 +1148,6 @@ static void stats_average_stddev_min_max(
 			}
 			stddev->value[j] = total / (double)num;
 			stddev->value[j] = sqrt(stddev->value[j]);
-
-			geometric_mean->value[j] = (num == 0) ? 0.0 :
-				pow(geometric_mean->value[j], 1.0 / (double)num);
 		} else {
 			average->inaccurate[j] = true;
 			max->inaccurate[j] = true;
@@ -1462,6 +1471,11 @@ static int power_get_sys_fs(
 			return 0;
 		}
 		(void)printf("Device is not discharging, cannot measure power usage.\n");
+		(void)printf("Perhaps re-run with -z (ignore zero power)"
+#if defined(POWERSTAT_X86)
+			     " or -R (RAPL)"
+#endif
+			     "\n");
 		return -1;
 	}
 
@@ -2309,6 +2323,7 @@ static void cpu_states_dump(void)
 	cpu_state_t *s;
 	double c0_percent = 100.0;
 	bool c0 = false;
+	int name_width = 10;
 
 	if (!cpu_states_list)
 		return;
@@ -2317,6 +2332,10 @@ static void cpu_states_dump(void)
 		cpu_info_t *ci;
 		uint64_t state_total = 0;
 		double time_total = 0;
+		const int len = strlen(s->name);
+
+		if (len > name_width)
+			name_width = len;
 
 		for (ci = s->cpu_info_list; ci; ci = ci->list_next) {
 			state_total += ci->time_diff;
@@ -2333,13 +2352,13 @@ static void cpu_states_dump(void)
 	}
 
 	if (c0_percent >= 0.0) {
-		(void)printf("\n%-10s %7s %10s %-8s\n",
-			"C-State", "Resident", "Count", "Latency");
+		(void)printf("\n%-*s %7s %10s %-8s\n",
+			name_width, "C-State", "Resident", "Count", "Latency");
 		for (s = cpu_states_list; s; s = s->list_next)
-			(void)printf("%-10s %7.3f%% %10" PRIu64 "%8" PRIu64 "\n",
-				s->name, s->resident, s->usage_total, s->latency);
+			(void)printf("%-*s %7.3f%% %10" PRIu64 "%8" PRIu64 "\n",
+				name_width, s->name, s->resident, s->usage_total, s->latency);
 		if (!c0)
-			(void)printf("%-10s %7.3f%%\n", "C0", c0_percent);
+			(void)printf("%-*s %7.3f%%\n", name_width, "C0", c0_percent);
 	} else {
 		(void)printf("\nUntrustworthy C-State states, ignoring.\n");
 	}
@@ -2717,7 +2736,7 @@ static int monitor(const int sock)
 					proc_info_add(proc_ev->event_data.fork.child_pid);
 					if (opts & OPTS_SHOW_PROC_ACTIVITY) {
 						char *type = is_thread ? "clone" : "fork";
-               					log_printf("fork: parent pid=%d -> %s pid=%d (%s)\n",
+               					(void)log_printf("fork: parent pid=%d -> %s pid=%d (%s)\n",
 							ppid,
 							type,
                        					proc_ev->event_data.fork.child_pid,
@@ -2728,7 +2747,7 @@ static int monitor(const int sock)
            			case PROC_EVENT_EXEC:
 					stats[readings].value[PROC_EXEC] += 1.0;
 					if (opts & OPTS_SHOW_PROC_ACTIVITY) {
-               					log_printf("exec: pid=%d (%s)\n",
+               					(void)log_printf("exec: pid=%d (%s)\n",
                        					proc_ev->event_data.exec.process_tgid,
 							proc_info_get(proc_ev->event_data.exec.process_pid));
 					}
@@ -2737,7 +2756,7 @@ static int monitor(const int sock)
            			case PROC_EVENT_EXIT:
 					stats[readings].value[PROC_EXIT] += 1.0;
 					if (opts & OPTS_SHOW_PROC_ACTIVITY) {
-               					log_printf("exit: pid=%d exit_code=%d (%s)\n",
+               					(void)log_printf("exit: pid=%d exit_code=%d (%s)\n",
                         				proc_ev->event_data.exit.process_pid,
                         				proc_ev->event_data.exit.exit_code,
 							proc_info_get(proc_ev->event_data.exit.process_pid));
@@ -2837,7 +2856,7 @@ static int monitor(const int sock)
 			stats_histogram(stats, readings, CPU_FREQ,
 				"CPU average frequencies",
 				"Histogram (of %d CPU average frequencies)\n\n",
-				"Range (GHz)", 1e9);
+				"Range (GHz)", 1000.0);
 		if (opts & OPTS_THERMAL_ZONE) {
 			uint8_t	tz;
 
